import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

## Lambda options
lam=np.array([0.001,0.01,0.05,0.1,0.2,0.3,0.4,0.5,0.6,0.65])
gamma=5/3

## Auxiliar Function l/l0 (lambda)
def ll0(l):
    p1 = -(2-gamma)/(2*(gamma-1))
    p2 = gamma/(2*(gamma-1))
    t  = 1-l/(gamma-1)
    pr1 = l**p1
    pr2 = t**p2
    return pr1*pr2

## Auxiliar Function e(P):
def ef(p,l):
    a=gamma-1-l
    b=a*p**(1/gamma)+l*p
    return b/(gamma-1)

## Auxiliar Function P'(x)
def PP(x,P0,w0,l):
    a = ef(P0,l)+l*P0
    b = w0+3*l*P0
    c = 1-2*l*x*x*w0
    return -a*x*b/c
    
## Auxiliar Function omega'(x)    
def WW(x,P0,w0,l):
    if x==0:
        return 0
    else:
        return -(3/x)*(w0-ef(P0,l))

# RK4 method
def rk4(x0,P0,W0,l):
    Psol=[]
    Wsol=[]
    x=[]
    # Step size
    h = 0.0005
    while P0>=0:
        k1  = h * (PP(x0, P0,W0,l))
        kk1 = h * (WW(x0, P0,W0,l))
        
        k2  = h * PP(x0+h/2, P0+k1/2, W0+kk1,l)
        kk2 = h * WW(x0+h/2, P0+k1/2, W0+kk1,l)
        
        k3  = h * PP(x0+h/2, P0+k2/2, W0+kk2,l)
        kk3 = h * WW(x0+h/2, P0+k2/2, W0+kk2,l)
        
        k4  = h * PP(x0+h, P0+k3, W0+kk3,l)
        kk4 = h * WW(x0+h, P0+k3, W0+kk3,l)
        
        k  = (k1  +2*k2  +2*k3  +k4 )/6
        kk = (kk1 +2*kk2 +2*kk3 +kk4)/6
        
        Pn = P0 + k
        Wn = W0 + kk
        
        Psol.append(P0)
        Wsol.append(W0)
        x.append(x0)
        P0 = Pn
        W0 = Wn
        x0 = x0+h
    return [np.array(x),np.array(Psol),np.array(Wsol)]

## Inputs
x0=0
P0=1
W0=1

## RK4 Running 
Rl =[]
Ml =[]
Rl0=[]
Ml0=[]
MR =[]
for i in range (0,len(lam)):
    xf=rk4(x0,P0,W0,lam[i])[0][-1]
    Wf=rk4(x0,P0,W0,lam[i])[2][-1]
    
    Rl.append(xf)
    Ml.append(lam[i]*Wf*xf**3)
    Rl0.append(ll0(lam[i])*xf)
    Ml0.append(ll0(lam[i])*lam[i]*Wf*xf**3)
    MR.append(2*lam[i]*Wf*xf**2)

print('Cargando Tabla')
## Table
Rlt =['{:.2f}'.format(x) for x in Rl ]
Mlt =['{:.4f}'.format(x) for x in Ml ]
Rl0t=['{:.2f}'.format(x) for x in Rl0]
Ml0t=['{:.4f}'.format(x) for x in Ml0]
MRt =['{:.5f}'.format(x) for x in MR ]
lam_tabla = ['{}'.format(x) for x in lam]
data = {'$R/l$':Rlt,'$M/l$':Mlt,
		'$R/l_0$':Rl0t,'$M/l_0$':Ml0t,'$2M/R$':MRt}
df = pd.DataFrame(data,index=lam_tabla)
df.columns.name = '$\lambda$'
print(df) 

## Plots
print('Cargando Figuras')
lam2=np.array([0.01,0.1,0.2,0.4,0.6])

r_0=ll0(lam2[0])*rk4(x0,P0,W0,lam2[0])[0]
P_0=rk4(x0,P0,W0,lam2[0])[1]
Y_0=2*lam2[0]*rk4(x0,P0,W0,lam2[0])[2]*(rk4(x0,P0,W0,lam2[0])[0])**2

r_1=ll0(lam2[1])*rk4(x0,P0,W0,lam2[1])[0]
P_1=rk4(x0,P0,W0,lam2[1])[1]
Y_1=2*lam2[1]*rk4(x0,P0,W0,lam2[1])[2]*(rk4(x0,P0,W0,lam2[1])[0])**2

r_2=ll0(lam2[2])*rk4(x0,P0,W0,lam2[2])[0]
P_2=rk4(x0,P0,W0,lam2[2])[1]
Y_2=2*lam2[2]*rk4(x0,P0,W0,lam2[2])[2]*(rk4(x0,P0,W0,lam2[2])[0])**2

r_3=ll0(lam2[3])*rk4(x0,P0,W0,lam2[3])[0]
P_3=rk4(x0,P0,W0,lam2[3])[1]
Y_3=2*lam2[3]*rk4(x0,P0,W0,lam2[3])[2]*(rk4(x0,P0,W0,lam2[3])[0])**2

r_4=ll0(lam2[4])*rk4(x0,P0,W0,lam2[4])[0]
P_4=rk4(x0,P0,W0,lam2[4])[1]
Y_4=2*lam2[4]*rk4(x0,P0,W0,lam2[4])[2]*(rk4(x0,P0,W0,lam2[4])[0])**2

plt.rc('xtick', labelsize=13) 
plt.rc('ytick', labelsize=13) 
fig, (ax1, ax2) = plt.subplots(1, 2)
fig.set_figheight(6)
fig.set_figwidth(16)

##################### Figure 1 ################
ax1.plot(r_0, P_0, label='$\lambda=0.01$' ,color='navy')
ax1.plot(r_1, P_1, label='$\lambda=0.1$' ,color='blueviolet')
ax1.plot(r_2, P_2, label='$\lambda=0.2$' ,color='darkred')
ax1.plot(r_3, P_3, label='$\lambda=0.4$' ,color='darkgreen')
ax1.plot(r_4, P_4, label='$\lambda=0.6$' ,color='darkslategray')
ax1.set_xlabel('$r/l_0$',fontsize=14)
ax1.set_ylabel('$p/p_c$',fontsize=14)
ax1.legend(loc="upper right",fontsize=14,fancybox=True,shadow=True)
ax1.grid(ls='--')

##################### Figure 2 ################
ax2.plot(r_0, Y_0, label='$\lambda=0.01$' ,color='navy')
ax2.plot(r_1, Y_1, label='$\lambda=0.1$' ,color='blueviolet')
ax2.plot(r_2, Y_2, label='$\lambda=0.2$' ,color='darkred')
ax2.plot(r_3, Y_3, label='$\lambda=0.4$' ,color='darkgreen')
ax2.plot(r_4, Y_4, label='$\lambda=0.6$' ,color='darkslategray')
ax2.set_xlabel('$r/l_0$',fontsize=14)
ax2.set_ylabel('$2m(r)/r$',fontsize=14)
ax2.legend(loc="upper right",fontsize=14,fancybox=True,shadow=True)
ax2.grid(ls='--')
#plt.savefig('fig33.pdf')
plt.show()

